@@ -15,6 +15,7 @@ from datetime import datetime
 
 from ..utils.io import load_data
 from ..utils import validator
+from ..utils.config import get_config_or_params
 
 
 class BaseTable:
@@ -24,16 +25,26 @@ class BaseTable:
     Provides common functionality for loading data, running validations,
     and generating reports. All table-specific classes should inherit from this.
     
-    Attributes:
-        data_directory (str): Path to the directory containing data files
-        filetype (str): Type of data file (csv, parquet, etc.)
-        timezone (str): Timezone for datetime columns
-        output_directory (str): Directory for saving output files and logs
-        table_name (str): Name of the table (from class name)
-        df (pd.DataFrame): The loaded data
-        schema (dict): The YAML schema for this table
-        errors (List[dict]): Validation errors from last validation run
-        logger (logging.Logger): Logger for this table
+    Attributes
+    ----------
+    data_directory : str
+        Path to the directory containing data files
+    filetype : str
+        Type of data file (csv, parquet, etc.)
+    timezone : str
+        Timezone for datetime columns
+    output_directory : str
+        Directory for saving output files and logs
+    table_name : str
+        Name of the table (from class name)
+    df : pd.DataFrame
+        The loaded data
+    schema : dict
+        The YAML schema for this table
+    errors : List[dict]
+        Validation errors from last validation run
+    logger : logging.Logger
+        Logger for this table
     """
     
     def __init__(
@@ -47,13 +58,19 @@ class BaseTable:
         """
         Initialize the BaseTable.
         
-        Parameters:
-            data_directory (str): Path to the directory containing data files
-            filetype (str): Type of data file (csv, parquet, etc.)
-            timezone (str): Timezone for datetime columns
-            output_directory (str, optional): Directory for saving output files and logs.
-                If not provided, creates an 'output' directory in the current working directory.
-            data (pd.DataFrame, optional): Pre-loaded data to use instead of loading from file
+        Parameters
+        ----------
+        data_directory : str
+            Path to the directory containing data files
+        filetype : str
+            Type of data file (csv, parquet, etc.)
+        timezone : str
+            Timezone for datetime columns
+        output_directory : str, optional
+            Directory for saving output files and logs.
+            If not provided, creates an 'output' directory in the current working directory.
+        data : pd.DataFrame, optional
+            Pre-loaded data to use instead of loading from file
         """
         # Store configuration
         self.data_directory = data_directory
@@ -147,49 +164,79 @@ class BaseTable:
     @classmethod
     def from_file(
         cls, 
-        data_directory: str,
-        filetype: str,
-        timezone: str = "UTC",
+        data_directory: Optional[str] = None,
+        filetype: Optional[str] = None,
+        timezone: Optional[str] = None,
+        config_path: Optional[str] = None,
         output_directory: Optional[str] = None,
         sample_size: Optional[int] = None,
         columns: Optional[List[str]] = None,
         filters: Optional[Dict[str, Any]] = None
-    ):
+    ) -> 'BaseTable':
         """
         Load data from file and create a table instance.
         
-        Parameters:
-            data_directory (str): Path to the directory containing data files
-            filetype (str): Type of data file (csv, parquet, etc.)
-            timezone (str): Timezone for datetime columns (default: UTC)
-            output_directory (str, optional): Directory for saving output files and logs
-            sample_size (int, optional): Number of rows to load
-            columns (List[str], optional): Specific columns to load
-            filters (Dict, optional): Filters to apply when loading
+        Parameters
+        ----------
+        data_directory : str, optional
+            Path to the directory containing data files
+        filetype : str, optional
+            Type of data file (csv, parquet, etc.)
+        timezone : str, optional
+            Timezone for datetime columns
+        config_path : str, optional
+            Path to configuration JSON file
+        output_directory : str, optional
+            Directory for saving output files and logs
+        sample_size : int, optional
+            Number of rows to load
+        columns : List[str], optional
+            Specific columns to load
+        filters : Dict, optional
+            Filters to apply when loading
             
-        Returns:
+        Notes
+        -----
+        Loading priority:
+            1. If all required params provided → use them
+            2. If config_path provided → load from that path, allow param overrides
+            3. If no params and no config_path → auto-detect config.json
+            4. Parameters override config file values when both are provided
+            
+        Returns
+        -------
+        BaseTable
             Instance of the table class with loaded data
         """
+        # Get configuration from config file or parameters
+        config = get_config_or_params(
+            config_path=config_path,
+            data_directory=data_directory,
+            filetype=filetype,
+            timezone=timezone,
+            output_directory=output_directory
+        )
+        
         # Derive snake_case table name from PascalCase class name
         table_name = ''.join(['_' + c.lower() if c.isupper() else c for c in cls.__name__]).lstrip('_')
         
         # Load data using existing io utility
         data = load_data(
             table_name, 
-            data_directory, 
-            filetype, 
+            config['data_directory'], 
+            config['filetype'], 
             sample_size=sample_size,
             columns=columns,
             filters=filters,
-            site_tz=timezone
+            site_tz=config['timezone']
         )
         
         # Create instance with loaded data
         return cls(
-            data_directory=data_directory,
-            filetype=filetype,
-            timezone=timezone,
-            output_directory=output_directory,
+            data_directory=config['data_directory'],
+            filetype=config['filetype'],
+            timezone=config['timezone'],
+            output_directory=config.get('output_directory', output_directory),
             data=data
         )
     
@@ -198,6 +245,7 @@ class BaseTable:
         Run comprehensive validation on the data.
         
         This method runs all validation checks including:
+        
         - Schema validation (required columns, data types, categories)
         - Missing data analysis
         - Duplicate checking
